package com.example.catchTheLetters.service.impl;

import com.example.catchTheLetters.entity.Level;
import com.example.catchTheLetters.entity.ScoreInfo;
import com.example.catchTheLetters.model.vo.RankVo;
import com.example.catchTheLetters.service.AuthService;
import com.example.catchTheLetters.service.LevelService;
import com.example.catchTheLetters.utils.JwtUtil;
import com.example.catchTheLetters.utils.R;
import jakarta.annotation.Resource;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import com.mongodb.client.result.DeleteResult;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author 慕华
 * @date 2024/4/10
 * @Version 1.0
 * @description
 */
@Service
public class LevelServiceImpl implements LevelService {

    @Resource
    private MongoTemplate mongoTemplate;

    @Resource
    private AuthService authService;

    @Override
    public R<Level> levelDetail(String id) {
        Level level = mongoTemplate.findOne(new Query(Criteria.where("id").is(id)), Level.class);
        if (level == null) {
            return R.fail("查询失败，请重试");
        }
        return R.ok(level);
    }

    @Override
    public R<List<Level>> list() {
        Query query = new Query();
        query.fields().include("id").include("name").include("type").include("order");
        List<Level> levels = mongoTemplate.find(query, Level.class);
        return R.ok(levels);
    }

    @Override
    public R<ScoreInfo> settle(ScoreInfo scoreInfo, String token) {
        var user = authService.verify(token);
        if (user == null) {
            return R.fail("token解析失败");
        }
        if (user.getId().equals(scoreInfo.getUserId())) {
            // 比对这一关是否是用户的最新进度，如果是，就更新用户的进度
            var levelRes = levelDetail(scoreInfo.getLevelId());
            if (levelRes.getCode() == R.FAIL) {
                return R.fail("查询关卡失败，请重试");
            }
            if (user.getProgress() == null || levelRes.getData().getOrder() > user.getProgress()) {
                user.setProgress(levelRes.getData().getOrder());
                // 这里要直接存，因为authService的update方法没有修改progress字段
                mongoTemplate.save(user);
            }

            // 先获取数据库中的分数，如果有则比较，如果没有则直接插入
            var score = mongoTemplate.findOne(new Query(Criteria.where("userId").is(scoreInfo.getUserId()).and("levelId").is(scoreInfo.getLevelId())), ScoreInfo.class);
            ScoreInfo insert = new ScoreInfo();
            if (score != null) {
                if (score.getScore() < scoreInfo.getScore()) {
                    score.setScore(scoreInfo.getScore());
                    score.setTime(scoreInfo.getTime());
                    insert = mongoTemplate.save(score);
                }
            } else {
                insert = mongoTemplate.save(scoreInfo);
            }

            return R.ok(insert);
        }
        return R.fail("添加失败，请重试");
    }

    @Override
    public RankVo getRankInfo(String levelId, String token) {
        // 在ScoreInfo表当中，查询levelId为levelId的所有数据，按照分数降序排列，取前10条数据
        var info = mongoTemplate.find(new Query(Criteria.where("levelId").is(levelId)).limit(10), ScoreInfo.class);

        var rankVo = new RankVo();
        rankVo.setScoreInfos(info);


        // 如果有token，就查自己的分数数据
        if (token != null) {
            var map = JwtUtil.getPayload(token);
            String id = null;
            if (map != null) {
                id = map.get("id").asString().replaceAll("\"", "");
            }
            if (id != null) {
                var myScore = mongoTemplate.findOne(new Query(Criteria.where("userId").is(id).and("levelId").is(levelId)), ScoreInfo.class);
                rankVo.setMyScore(myScore);
            }
        }

        return rankVo;
    }

    @Override
    public boolean levelCreate(Level level) {
        var insert = mongoTemplate.insert(level);
        return insert.getId() != null;
    }

    @Override
    public boolean delete(String id) {
        Level level = new Level();
        level.setId(id);
        DeleteResult deleteRequest = mongoTemplate.remove(level);
        return deleteRequest.getDeletedCount() > 0;
    }

    @Override
    public boolean update(Level level) {
        var save = mongoTemplate.save(level);
        return save.getId() != null;
    }
}
